// *************************************
// * Scamnum Specularis ** 20. 9. 2006 *
// *************************************

#include ".\menic.h"

#define  TXo PIN_B5           // To the transmitter modulator
#include ".\AX25.c"           // Podprogram pro prenos telemetrie
char AXstring[40];            // Buffer pro prenos telemetrie

#define REFSTART  95    // Hodnota odpovidajici 1,2V pri nezatizenych clancich
#define DUTY1     2     // Delka sepnuti PWM v us pro sero
#define DUTY2     19    // Delka sepnuti PWM v us pro jednu zarovku 500W

// Tabulka hodnot prevodniku pro ruzna napajeci napeti
//  80   3,99V
//  90   3,54V
// 100   3,20V
// 105   3,04V
// 107   2,99V
// 110   2,91v
// 200   1,61V

#define PIEZO     PIN_A6   // Jeden pol piezo reproduktoru
#define LED       PIN_A7   // LED / druhy pol piezo reproduktoru
#define MENIC     PIN_B3   // Spinani tranzistoru menice
#define REFPWR    PIN_B1   // Napajeni zdroje referencniho napeti
#define MOTOR     PIN_B2   // Spinani tranzistoru motoru
#define SW1       PIN_B6   // DIPswitch 1
#define SW2       PIN_B7   // DIPswitch 2
#define NAPETI    4        // Cislo kanalu DA prevodniku pro mereni napeti
#define PWM       0        // Cislo kanalu DA prevodniku pro nastaveni PWM
#define ROZJEZD   1        // Cislo kanalu DA prevodniku pro nastaveni rozjezdu

#byte SSPBUF   = 0x13   // Adresy registru jednotky SSP
#byte SSPCON1  = 0x14
#byte SSPSTAT  = 0x94

// Vzorky pro generovani PWM pomoci SSP
unsigned int8 const Pattern[8] = {0x02,0x06,0x0e,0x1e,0x3e,0x7e,0xfe,0xff};
unsigned int8 MotorPattern;

#INT_SSP
void IntSSP()
{
   SSPBUF=MotorPattern;             // znovu vysli stejnou hodnotu
}

void pip()     // pipnuti a bliknuti
{
   int1 beep;
   int i;

   for(i=0; i<250; i++)
   {
      output_bit(PIEZO, beep);
      beep=~beep;
      output_bit(LED, beep);
      delay_us(100);
   };
   output_high(PIEZO);
   output_high(LED);
}

void cvak()    // pouze cvaknuti piezoelementem bez bliknuti
{
   output_bit(PIEZO,~input(PIEZO));
}

void main()
{
   int8 n;
   int8 perioda;
   int8 razeni;
   int8 uclanku;
   int16 ble;

   setup_timer_0(RTCC_INTERNAL);setup_wdt(WDT_1152MS);
   setup_timer_1(T1_DISABLED);
   setup_adc_ports(ADC_CLOCK_INTERNAL|sAN0|sAN1|sAN4|VSS_VDD);
   setup_adc(ADC_CLOCK_INTERNAL);
   setup_spi(FALSE);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);

   output_low(MENIC);      // Vsechno zastav
   output_low(MOTOR);
   output_low(REFPWR);
   output_high(LED);
   output_high(PIEZO);

   set_adc_channel(NAPETI);     // Vyber AD kanal pro pomerne mereni napeti
   set_pwm1_duty(0);       // Spust PWM, ale zatim s trvalou 0 na vystupu
   setup_ccp1(CCP_PWM);
   setup_timer_2(T2_DIV_BY_1,100,1);  // perioda

   setup_oscillator(OSC_4MHZ|OSC_INTRC);  // rozjed PIC (zvysi se odber)

// jsme v depu?
   set_pwm1_duty(DUTY1);          // PWM pro zatizeni clanku pri malem osvetleni
   n=0;
   do
   {
      output_low(REFPWR);  // Vypni napajeni napetove reference, aby se setrilo
      delay_ms(100);
      if (4==n++) {cvak(); n=0;};   // Kazdy 4. pruchod cvakni
      output_high(REFPWR);          // Zapni napajeni napetove reference
      delay_ms(1);                  // Pockej na ustaleni napeti
   } while (read_adc()>REFSTART);   // Odkryl starter clanky?
   output_low(REFPWR);     // Vypni napajeni napetove reference, aby se setrilo

//   port_b_pullups(TRUE);            // Zapni pullupy, aby slo cist DIPswitch
//   do
//   {
      // Precti trimry
      set_adc_channel(PWM);         // Zjisteni nastaveni PWM
      delay_us(100);                // Pockej na prepnuti kanalu A/D prevodniku
      perioda=(read_adc()>>2)+40;   // rozsah: 40 az 103

      set_adc_channel(ROZJEZD);     // Zjisteni nastaveni rozjezdu
      delay_us(100);                // Pockej na prepnuti kanalu A/D prevodniku
      razeni=read_adc()>>3;         // rozsah: 0 az 31

      // Rozjeti menice
      set_pwm1_duty(0);    // Zastav PWM, aby slo nastavit jinou frekvenci
      setup_timer_2(T2_DIV_BY_1,perioda,1);    // perioda
      set_pwm1_duty(DUTY2); // PWM pro zatizeni clanku pri jedne 500W zarovce

      pip();                  // potvrzeni odstartovani / servisni mod
      delay_ms(500);

//   } while (input(SW1));      // Je zapnut servisni mod? SW1(OFF)=ANO
//   port_b_pullups(FALSE);     // Vypni pullupy, aby se setrilo energii

   // zmereni napeti na clancich
   output_high(REFPWR);          // Prepni AD na mereni napeti clanku
   set_adc_channel(NAPETI);      // Prepni Zmer napeti clanku
   delay_us(50);                // Pockej na prepnuti kanalu A/D prevodniku
   uclanku=read_adc();
   output_low(REFPWR);     // odpoj odpor pro mereni napeti

   // posli telemetrii
   sprintf(AXstring,"p=%u r=%u u=%u", perioda, razeni, uclanku);  // Convert DATA to String.
   SendPacket(&AXstring[0]);

   delay_ms(14000);           // Pockame do 14.5s

   // rozjezd
   for(ble=1;ble<=1000;ble++)
   {   
      for(n=0;n<=razeni;n++)
      {
         output_high(MOTOR);              
         delay_us(ble);   
         output_low(MOTOR);             
         delay_us(1000-ble);   
      }
   }

// jedem co to da
   output_high(MOTOR);

   delay_ms(1300);                  // Nemeli bysme jet dele nez 1,3s :-)
   output_low(MOTOR);               // Zastav motor
                                    // Menic je stale zapnut a zatezuje clanky

   while(TRUE);            // Cekej, dokud se neztrati svetlo,
                           // nebo prijde WatchDog

}
